<?php

namespace Drupal\page_deletion_guard\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Builds admin form to lock and display locked pages.
 */
class PageDeletionGuardForm extends ConfigFormBase {

  /**
   * Constructs a PageDeletionGuardForm object.
   *
   * @param \Drupal\node\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'page_deletion_guard_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['page_deletion_guard.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $options = [];
    $config = $this->config('page_deletion_guard.settings');
    $locked_nodes = $config->get('locked_pages.nodes') ?: [];

    if (!empty($locked_nodes)) {
      $nodes = $this->entityTypeManager->getStorage('node')->loadMultiple($locked_nodes);
      foreach ($nodes as $node) {
        $options[$node->id()] = $node->getTitle();
      }
    }

    $form['entity_reference'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Search and select pages to lock:'),
      '#target_type' => 'node',
      '#tags' => TRUE,
      '#description' => $this->t("Can select multiple pages, separated by a comma."),
      '#maxlength' => 512,
    ];

    $form['locked_pages'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Currently locked pages:'),
      '#options' => $options,
      '#default_value' => $locked_nodes,
      '#description' => $this->t('Deselect pages to unlock them.'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $selected_nodes = $form_state->getValue('entity_reference');
    $selected_node_ids = [];

    if (!empty($selected_nodes)) {
      foreach ($selected_nodes as $node) {
        if (is_array($node) && isset($node['target_id'])) {
          $selected_node_ids[] = $node['target_id'];
        }
      }
    }

    $locked_pages = array_filter($form_state->getValue('locked_pages')) ?: [];
    $locked_pages_combined = array_unique(array_merge($selected_node_ids, array_keys($locked_pages)));

    $this->config('page_deletion_guard.settings')
      ->set('locked_pages.nodes', $locked_pages_combined)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
